package DBIx::Interpolate;

our $VERSION = '0.2';

=head1 NAME

DBIx::Interpolate-0.2 - Wrapper for DBI to interpolate variables into SQL

=head1 SYNOPSIS

  use DBIx::Interpolate;
  
  my $dbh = DBIx->connect($data_source, $username, $auth, \%attr);
  my $rows = $dbh->do(qq[
      SELECT * FROM table
      WHERE color IN], \@colors, qq[
            AND y = ], \$x,
      'LIMIT', \(1), ',', \(10)
  );
  
  # or when using SQL::Interpolate with source filtering enabled:
  
  use DBIx::Interpolate;
  use SQL::Interpolate FILTER => 1, qw(:all);
  
  my $dbh = DBIx->connect($data_source, $username, $auth, \%attr);
  my $rows = $dbh->do(sql[
      SELECT * FROM table
      WHERE color IN @colors
            AND y = $x
      LIMIT(start => 1, count => 10)
  ]);

=head1 DESCRIPTION

This module is a convenience module that wraps around DBI in order to
transparently support SQL::Interpolate.  All methods are designed to
behave identically to their DBI counterpart except that certain
methods, such as the do, select.*, and prepare.* methods, expect a
list of input arguments in the format accepted by
SQL::Interpolate::sql_interp.  This allows you to write

  $dbh->fetchall_arrayref(qq[
      SELECT * from mytable
      WHERE height > ], \$x
  );

rather than using the dbi_interp "adaptor" function as in

  $dbh->fetchall_arrayref(dbi_interp $dbh, qq[
      SELECT * from mytable
      WHERE height > ], \$x
  );

Internally, this module will pass the database handle to
the adapter function so that the generate SQL will be specific
to the SQL dialect of your database (e.g. LIMIT clause syntax).

You might also wish to use this module with source filtering
enabled on the SQL::Interpolate module in order to instead use
the sql// quote-like operator:

  $dbh->fetchall_arrayref( sql[
      SELECT * from mytable
      WHERE height > $x
  ] );

=head1 CREDITS

This module and its design were originally suggested by Jim Cromie.

=head1 SEE ALSO

L<SQL::Interpolate>, DBI

=head1 AUTHOR

David Manura (http://math2.org/david).

Bug reports and comments on the design are most welcome.

=head1 COPYRIGHT

  Copyright (c) 2004, David Manura. All Rights Reserved.
  This module is free software. It may be used, redistributed
  and/or modified under the same terms as Perl itself.

=cut

use strict;
use warnings;
use DBI;
use Carp;
use SQL::Interpolate qw(:all);

sub AUTOLOAD
{
    my($self, @args) = @_;
    (my $meth = our $AUTOLOAD) =~ s/.*:://;
    #print "[$meth]";
    return unless $meth =~ /[^A-Z]/; # e.g. DESTROY

    my @x;
    my $want = wantarray();

    eval {
        my @new_args;

        if($meth =~ /^( do | select.* | prepare.* )$/x) {
            @new_args = dbi_interp($$self, @args);
        }
        else {
            @new_args = @args;
        }

        # Is there a simpler way?
        if($want)             { @x    = $$self->$meth(@new_args); }
        elsif(defined($want)) { $x[0] = $$self->$meth(@new_args); }
        else                  {         $$self->$meth(@new_args); }
    };
    if($@) { croak $@; }

    return @x    if $want;
    return $x[0] if defined($want);
    return;
}

sub connect
{
    my $class = shift;
    my $self;
    eval {
        my $dbh = DBI->connect(@_);
        return if !$dbh;
        $self = bless \$dbh, __PACKAGE__;
    };
    if($@) { croak $@; }
    return $self;
}
