# Interpolate.pm
#
# Copyright (c) 2003 David Manura
# This file is licensed under the same terms as Perl itself.

our $VERSION = "0.2";

=head1 NAME

SQL::Interpolate - Simplified interpolation of variables into SQL
                   statements.

=head1 SYNOPSIS

  #-- Examples without source filtering.
  
  use DBI;
  use SQL::Interpolate qw(:all);
  
  my @colors = ('blue', 'green');
  my $rows = $dbh->selectall_arrayref(dbi_interp qq[
      SELECT * FROM table
      WHERE color IN], \@colors, qq[
            AND y = ], \$x,
      &limit(start => 1, count => 10)
  );
  
  # The above is equivalent to (assuming Postgres LIMIT syntax)
  #
  #   my $rows = $dbh->selectall_arrayref(qq[
  #       SELECT * FROM table
  #       WHERE color IN (?, ?)
  #             AND y = ?
  #       LIMIT ? OFFSET ?], undef, @colors, $x, 10, 1
  #   );
  
  $dbh->do(dbi_interp
      "INSERT INTO table", {color => $new_color,
                            shape => $new_shape}
  );
  
  # The above is equivalent to
  #
  #   $dbh->do(qq[
  #       INSERT INTO table (color, shape) VALUES(?, ?)],
  #       undef, $new_color, $new_shape
  #   );
  
  $dbh->do(dbi_interp qq[
      UPDATE table SET ],
          {color => $new_color,
           shape => $new_shape}, qq[
      WHERE color <> ], \$color
  );
  
  # The above is equivalent to
  #
  #   $dbh->do(qq[
  #       UPDATE table SET color = ?, shape = ?
  #       WHERE color <> ?], undef, $new_color, $new_shape
  #   );
  
  # Example using the SQL and bind values directly:
  my($sql, @bind) = sql_interp qq[
      SELECT * FROM table WHERE color IN], \@colors, qq[
          AND y = ], \$x
  );
  $d->prepare($sql);
  $d->execute(@bind);
  
  
  #-- Example with source filtering enabled
  #   (where it really gets interesting):
  
  use SQL::Interpolate FILTER => 1, qw(:all);
  
  my($sql, @bind) = sql_interp sql[
      SELECT * FROM table
      WHERE color IN @colors
            OR color IN ['green', 'red', @colors]
            AND color IN $rcolors
            AND d = $x
      LIMIT(start => $start, count => $count*2)
  ];
  
  # The above is equivalent to
  #
  #   my($sql, @bind) = sql_interp qq[
  #       SELECT * FROM table
  #       WHERE color IN ], \@colors, qq[
  #             OR color IN ], ['green', 'red', @colors], qq[
  #             AND color IN ], \$rcolors, qq[
  #             AND d = ], \$x,
  #       &limit(start => $start, count => $count*2)
  #   ;
  
  # Since sql// objects have a string concatenation operator,
  # the above is also equivalent to
  my($sql, @bind) = sql_interp sql[
      SELECT * FROM table
      WHERE color IN] . \@colors . sql[
            OR color IN] . ['green', 'red', @colors] . sql[
            AND color IN] . $rcolors . qq[
            AND d = ] . \$x .
      &limit(start => $start, count => $count*2)
  ;
  
  # After source filtering, this internally gets translated to
  #
  #   my($sql, @bind) = sql_interp
  #         new SQL::Interpolate::SQL("SELECT * FROM table WHERE color IN")
  #       . \@colors
  #       . new SQL::Interpolate::SQL("OR color IN")
  #       . ['green', 'red', @colors]
  #       . new SQL::Interpolate::SQL("AND color IN")
  #       . \$rcolors
  #       . "AND d = "
  #       . \$x
  #       . &limit(start => $start, count => $count*2)
  #   ;

=head1 DESCRIPTION

This module provides a convenient syntax for interpolating variables
into SQL statements.  SQL::Interpolate enables you to do this:

  my $rows = $dbh->selectall_arrayref(dbi_interp qq[
      SELECT thid, date_modified, suid, title, state, num_messages
      FROM threads
      WHERE state <> ], \$state, qq[AND suid IN], \@suids, qq[
      ORDER BY t.date_modified DESC
      LIMIT], &limit(start => $page*$count, count => $count)
  );

or even this (using the source filter to enable to quote-like sql//
operator and DBIx::Interpolate):

  my $rows = $dbh->selectall_arrayref( sql[
      SELECT thid, date_modified, suid, title, state, num_messages
      FROM threads
      WHERE state <> $state AND suid IN @suids
      ORDER BY t.date_modified DESC
      LIMIT(start => $page*$count, count => $count)
  ] );

rather than

  my $where .=
      '(0 ' . join('', map {" OR suid = " . $dbh->quote($_)} @suid) . ')';
  my $rows = $dbh->selectall_arrayref(qq(
      SELECT thid, date_modified, suid, title, state, num_messages
      FROM threads
      WHERE state <> ? AND $where
      ORDER BY t.date_modified DESC
      LIMIT ? OFFSET ?
  ), undef, $state, $count, $page*$count);

You can also do something like

  $dbh->do(dbi_interp qq[
      INSERT INTO table ],
          {color  => $new_color,
           shape  => $new_shape
           width  => $width,
           height => $height,
           length => $length    }
  );

or (with source filtering and DBIx::Interpolate):

  $dbh->do(sql[
      INSERT INTO table
          {color  => $new_color,
           shape  => $new_shape
           width  => $width,
           height => $height,
           length => $length    }
  ]);

or

  my $x = {color  => $new_color,
           shape  => $new_shape
           width  => $width,
           height => $height,
           length => $length    };
  $dbh->do(sql[INSERT INTO table $x]);

rather than

  $dbh->do(qq(
      INSERT INTO table (color, shape, width, height, length)
                  VALUES(?,     ?,     ?,     ?,      ?     )
  ), undef, $new_color, $new_shape, $width, $height, $length);

The purpose of SQL::Interpolate, therefore, is to make SQL statements
in Perl more natural, less redundant, and less error-prone.
SQL::Interpolate takes your query specification and generates
correctly formatted SQL statements B<with bind values>.  These result
values can then be passed to DBI or used for another purpose.
SQL::Interpolate serves a purpose similar to that of SQL::Abstract
except that SQL::Interpolate still exposes and utilizes the full
native SQL syntax of your database.

=head2 Security notes

SQL::Interpolate is designed to properly escape or bind variables.
One often sees people doing very unsafe things like this:

  use CGI;
  use DBI;
  ...
  my $q = new CGI;
  my $name = $q->param('name');
  $dbh->do("UPDATE mytable SET y = 0 WHERE x = '$name'");

The problem is that a malicious user could pass raw SQL into
$name (e.g. "one' or x = x or x ='")
so that your SQL statement might become

  $dbh->do("UPDATE mytable SET y = 0 WHERE x = 'one' or x = x or x=''");

or worse.  It's not a good idea to allow random web users to execute
arbitrary SQL on your database.

However, the sql// string-like operator does safely interpolate, so that

  $dbh->do(sql[UPDATE mytable SET y = 0 WHERE x = $name]);

or

  $dbh->do(sql[UPDATE mytable SET y = 0 WHERE x = ] . \$name);

(both using DBIx::Interpolate) is not the same thing as

  $dbh->do('UPDATE mytable SET y = 0 WHERE x = ' . $name);

but rather is equivalent to

  $dbh->do("UPDATE mytable SET y = 0 WHERE x = ?", undef, $name);

Be careful not to do this:

  $dbh->do(sql[UPDATE mytable SET y = 0 WHERE x = ] . $name);

because $name as a string (rather than a reference) will be recognized
as SQL rather than a variable to interpolate.  An sql// object
concatenated with a string will append the string verbatim into your
result SQL.  Future versions of SQL::Interpolate may throw an error if
one attempts to concatenate an sql// object with a string.  It's a
better idea to do this is you must:

  $dbh->do(sql[UPDATE mytable SET y = 0 WHERE x = ] . sql[$name]);

or simply

  $dbh->do(sql[UPDATE mytable SET y = 0 WHERE x = $name]);

because then the $name value will be properly interpolated.

=head2 Source filtering

To enable the quote-like sql// operator add a "FILTER => 1" to your use
statement as shown in the synopsis.  Just as it is possible to do with
q// or qq// operators, you can use various delimiters on the sql// operator,
such as

  sql[SELECT * from mytable WHERE x = $x]
  sql(SELECT * from mytable WHERE x = $x)
  sql<SELECT * from mytable WHERE x = $x>
  sql/SELECT * from mytable WHERE x = $x/

Internally, the sql operator returns an object of type SQL::Interpolate::SQL
such that any of the above is equivalent to

  SQL::Interpolate::SQL->new("SELECT * from mytable WHERE x = ", \$x)

This object comes with a string concatenation operator (.), you can
do things like

  sql[
    SELECT partnum, desc, price, stock
    FROM inventory
  ] . $show_all ? sql[] : sql[WHERE price > $price AND stock IN $stocks]

Disclaimer:
Source filtering is somewhat experimental because and takes some
work to implement due to the complexities of lexing Perl source code.
Therefore, I won't be surprised if sql// will fail on obscure syntax.

=head2 DBIx::Interpolate

DBIx::Interpolate can simplify using SQL::Interpolate and DBI
together.  What DBIx::Interpolate does is wrap DBI in order
that certain methods can take a list of interpolation expressions
instead of the usual ($sql_string, \%atts, @bind) signature.
Therefore, instead of

  $dbh->do(dbi_interp sql[DELETE FROM mytable WHERE x = $x])

you can omit the "dbi_interp" adaptor function and use

  $dbh->do(sql[DELETE FROM mytable WHERE x = $x]);

or

  $dbh->do("DELETE FROM mytable WHERE x =", \$x);

or even

  $dbh->do("DELETE FROM mytable WHERE x =", \$x, sql[OR y=$y]);

See the POD page on DBIx::Interpolate for details.

=head2 Macro functions

The sql// operator supports the inclusion of special macro calls.
These calls have an appearance similar to built-in SQL functions
(e.g. ADD, COUNT, MAX, MIN, etc.), but they are evaluated by the
sql// operator rather than by the database.  These special functions
serve as macros that preprocess your SQL.

Currently, the only supported such function is LIMIT, which provides a
wrapper around constructing SQL LIMIT clauses.  The syntax for LIMIT
clauses is not standard and varies by database, so using such a macro
can improve the portability of your code.  (Note: currently, this
macro generates MySQL-like LIMIT clauses for MySQL and Postgres-like
LIMIT/OFFSET clauses for everything else.  The full peculiarities of each
are not completely supported yet.  A more generic and robust mechanism
to transparently support these and other databases is expected in the
future, possibly through Terrence Brannon's SQL::AnyDBD module
currently in development.)

  my($sql, @bind) = sql_interp sql[
      SELECT * FROM table
      WHERE color IN @colors
            OR color IN ['green', 'red', @colors]
            AND color IN $rcolors
               AND d = $x
      LIMIT(start => $start, count => $count*2)
  ];

Note that the parameters can be arbitrary Perl expressions.  In fact,
internally, the above gets translated to

  my($sql, @bind) = sql_interp qq[
      SELECT * FROM table
      WHERE color IN ], \@colors, qq[
            OR color IN ], ['green', 'red', @colors], qq[
            AND color IN ], \$rcolors, qq[
               AND d = ], \$x,
      &SQL::Interpolate::limit(start => $start, count => $count*2), qq[
  ];

Note also how the interpolation of $start is very different from that
of $x.

"limit" above is a function exportable by this module that returns

  new SQL::Interpolate::Macro(
      type => 'limit', start => $start, count => $count*2)

This macro is expanded by sql_interp, and the expansion can
be dependent upon database if sql_interp is passed a database
handle as the first argument in its parameter list:

  my($sql, @bind) = sql_interp $dbh, qq[...]

See the below make_sql_interp and make_dbi_intep function calls
for information on using a closure to eliminate always passing a
database handle to obtain DB-specific SQL output.

=head1 Functions

=over 4

=item C<sql_interp>

  ($sql, @bind) = sql_interp(...);

This function strings together the passed parameters to generate a
properly formatted SQL statement containing bind ("?") parameters
along with the corresponding bind values in @bind.

To use sql_interp, pass it a list containing strings, variable
references, macros (i.e. SQL::Interpolate::Macro objects), and sql//
objects (i.e. SQL::Interpolate::SQL objects possibly themselves
containing other SQL strings, variable references for interpolation,
and macros).  Any sql// objects and macros passed in are first flattened
(via the sql_flatten function).  Then strings are
copied mostly verbatim into the output SQL (possibly with fine tweaking),
while the references are dereferenced and interpolated into
the SQL using binding ("?") placeholders, with corresponding values
pushed into @bind.

The first parameter may optionally be a database handle, in which
case sql_interp generates SQL specific to that SQL dialect.  For example,
the LIMIT clauses differ in syntax between databases.

Here is an example of invoking sql_interp:

  my @colors = ('blue', 'green');
  my($sql, @bind) = sql_interp qq[
      SELECT * FROM table WHERE color IN], \@colors, qq[
          AND y = ], \$x
  ;

which is equivalent to the following (bar whitespace):

  my $sql = "SELECT * FROM table WHERE color IN (?, ?) AND y = ?";
  my @bind = ('blue', 'green', $x);

Furthermore,

  my($sql, @bind) = sql_interp sql[
      SELECT * FROM table WHERE color IN @colors],
                                qq[ AND y = ], \$x
  ;

which is gets translated to

  my($sql, @bind) = sql_interp SQL::Interpolate::SQL->new(qq[
      SELECT * FROM table WHERE color IN ], \@colors),
                                qq[ AND y = ], \$x
  ;

also produces the same result as the above.

Note that variables must be passed as references (e.g. \$x) if not
using the sql// operator, otherwise they will recognized as SQL
fragments to be interpolated verbatim into the result SQL string.
Also, whitespaces are automatically added between parameters, so you
don't need to insert a extra spaces:

  sql_interp "INSERT INTO mytable", {x => 2}, "WHERE x IN", \@colors

gives the SQL

  "INSERT INTO mytable ? WHERE x in (?, ?)"

not

  "INSERT INTO mytable?WHERE x IN(?, ?)"

SQL::Interpolate will Do The Right Thing(TM) on trivial cases.  For
example, the above example will generate the following SQL if @colors
is an empty list:

  SELECT * FROM table WHERE shape = 'square' AND NULL

because the following mechanical answer is not valid SQL:

  SELECT * FROM table WHERE shape = 'square' AND color IN ()

SQL::Interpolate does not attempt to further optimize away such NULL
expressions since we can just as well let the database do this, as
databases are designed to do query optimization, without loss of
generality.

Furthermore, variable interpolation is affected by context.  So,

  sql_interp(qq[
      INSERT INTO mytable ], {color => 'blue', shape => 'square'}
  );

generates

  INSERT INTO mytable (color, shape) VALUES(?, ?)

whereas the following containing the same hash expression

  sql_interp(qq[
      UPDATE mytable SET ], {color => 'blue', shape => 'square'}
  );

generates SQL containing a different corresponding sub-expression:

  UPDATE mytable SET color = ?, shape = ?

.

=item C<dbi_interp>


  ($sql, $attr, @bind) = &dbi_interp(...);

This function is the same as C<sql_interp> except that is returns
an extra $attr value (set to undef) so that the return signature
matches the signature on many of the DBI module methods.  This allows
you to construct and execute an SQL statement in a single statement as
shown in the SYNPOSIS.  dbi_interp is unnecessary is use use
DBIx::Interpolate rather than the standard DBI module.

=item C<sql_flatten>

  @parts = sql_flatten(...);

This function flattens the contents of any sql// objects into
the containing list and processes any macros contained in the list.
This function takes the same type of input as sql_interp, and
sql_interp in fact uses this function to preprocess its input.
The result of this function is a list of SQL strings and
variable references for interpolation.

=item C<make_sql_interp>

  my $sql_interp = make_sql_interp($dbh, ...);

This creates a closure for the sql_interp function.  It is typically
used so that you don't have to always pass in a database handle into
sql_interp:

  my $interp = make_sql_interp($dbh);

  my($sql, @bind) = &$interp(sql[
     SELECT * FROM x WHERE y > $z
     LIMIT(start => $start, count => $count)
  ]);

=item C<make_dbi_interp>

  my $dbi_interp = make_dbi_interp($dbh, ...);

This is similar in purpose to make_sql_interp except that is generates
a closure around the dbi_interp function rather than the sql_interp
function.

=item C<limit>

  sql_interp q[SELECT * FROM mytable],
             &limit(start => $start, count => $count]

This function generates a LIMIT clause macro object (of type
SQL::Interpolate::Macro) suitable for inserting into an interpolation
list.  "start" is the index of the first row to return (1-indexed).
"count" is the maximum number of rows to return.  This limit object
can also be generated with the macro

  sql[LIMIT(start => $start, count => $count)]

=back

=head1 STATUS / TODO

This module is still B<under active design and development>.
Therefore, its interface remains very subject to change.
This module is not yet intended for production systems without
much caution.

Possible future additions include

=over 4

=item

Allow sql// operator to perform syntax checking on SQL
  (like a strict-mode).

=item

Allow DB-independent comments inside SQL.  These will get parsed
  out before sending to the database driver.

=back

=head1 SEE ALSO

L<DBIx::Interpolate>, DBI, SQL::AnyDBD

=head1 AUTHOR

David Manura (http://math2.org/david).

Bug reports and comments on the design are most welcome.

=head1 COPYRIGHT

Copyright (c) 2003-2004, David Manura.  All Rights Reserved.
This module is free software. It may be used, redistributed
and/or modified under the same terms as Perl itself.

=cut

package SQL::Interpolate;
use strict;
use base qw(Exporter);
use SQL::Interpolate::SQL;

our @EXPORT      = qw();
our %EXPORT_TAGS = (
    all => [qw( sql_interp
                dbi_interp
                make_sql_interp
                make_dbi_interp
                sql_flatten
                limit
               ) ]
);
our @EXPORT_OK = @{$EXPORT_TAGS{'all'}};

sub import
{
    my $class = shift;

    my $do_filter = 0;

    if(defined($_[0]) && $_[0] eq 'FILTER') {
        shift;
        $do_filter = shift;
    }
    unshift @_, $class;

    __PACKAGE__->export_to_level(1, @_);
    SQL::Interpolate::SQL->export_to_level(1);
    if($do_filter) {
        require SQL::Interpolate::Filter;
        goto &SQL::Interpolate::Filter::import; 
    }
}

# Flatten out SQL objects and macro objects in the interpolation list.
sub sql_flatten
{
    my(@parts  # list of SQL strings, variables references,
               #   SQL::Interpolate::SQL objects, and
               #   SQL::Interpolate::Macro objects.
               #   The first parameter may be a database handle.
    ) = @_;

    my $dbh = ref($parts[0]) eq 'DBI::db' ? (shift @parts) : undef;
    my $continue;
    do {
        $continue = 0;
        # flatten out and SQL::Interpolate::SQL objects
        @parts = map {
            if(ref($_) eq 'SQL::Interpolate::SQL') {
                $continue = 1;
                @{$_->{list}}
            }
            elsif(ref($_) eq 'SQL::Interpolate::Macro') {
                if($_->{type} eq 'limit') {
                    &make_limit($dbh, %$_);
                }
                else { die 'assert'; }
            } else { ($_) }
        } @parts;
    }
    while($continue);

    return @parts;
}

# Generate list of SQL strings and variable references from LIMIT macro.
sub make_limit
{
    my($dbh, %params) = @_;
    # FIX: generalize this and make more robust.
    #   Use SQL::AnyDBD.
    if(defined($dbh) && $dbh->{Driver}->{Name} eq 'mysql') {
        return ('LIMIT', \$params{start}, ',', \$params{count});
    }
    else {
        return ('LIMIT', \$params{count}, 'OFFSET', \$params{start});
    }
}


# Interpolate variables into SQL.
sub sql_interp
{
    my(@parts  # list of SQL strings, variable references,
               #   SQL::Interpolate::SQL objects, and
               #   SQL::Interpolate::Macro object.
               # The first parameter may be a database handle.
    ) = @_;

    @parts = sql_flatten @parts;

    my $sql = '';
    my $id = qr/[a-zA-Z_\.]+/;
    my @bind;
    foreach my $part (@parts) {

        # Note that sql[INSERT INTO mytable $x] gets translated to
        #   q[INSERT INTO mytable], \$x 
        # regardless whether $x is a scalar or reference since it
        # would be difficult to know at source filtering time whether
        # $x is already a reference.  Therefore, we dereference any
        # double reference here (at run-time).
        $part = $$part if ref($part) eq 'REF';

        if(ref($part)) {
            if(ref($part) eq 'SCALAR') {
                push @bind, $$part;
                $sql .= ' ? ';
            }
            elsif($sql =~ /\bIN\s*$/si && ref($part) eq 'ARRAY') {
                if(@$part == 0) {
                    $sql =~ s/$id\s+IN\s*$/NULL/s;
                }
                else {
                    push @bind, @$part;
                    $sql .= " (" . join(', ', map {'?'} @$part) . ") ";
                }
            }
            elsif($sql =~ /\bSET\s*$/si && ref($part) eq 'HASH') {
                die if keys %$part == 0;
                push @bind, values %$part;
                $sql .= " " . join(', ', map {"$_=?"} keys %$part)
                     .  " ";
            }
            elsif($sql =~ /\bINSERT\s+INTO\s*$id\s*$/si
                  && ref($part) eq 'HASH')
            {
                die if keys %$part == 0;
                push @bind, values %$part;
                $sql .= " (" . join(', ', keys %$part) . ")"
                     .  " VALUES(" . join(', ', map {'?'} keys %$part) . ")";
            }
            else { die "Unrecognized '$part'."; }
        }
        else {
            $sql .= " $part ";
        }
    }
    #print STDERR "[SQL:$sql /// " . join(':', @bind) . "]\n";
    return($sql, @bind);
}

# Interpolate variables into SQL statement.
# This is a wrapper function around sql_interp() that returns
#   also the \%attr value for using the result directly in the DBI
#   functions.
sub dbi_interp
{
    my($sql, @bind) = &sql_interp(@_);
    return($sql, undef, @bind);
}

# Create limit macro object.
# This may be inserted into the interpolation list.
# FIX: use SQL::AnyDBD to make database-independent.
sub limit
{
    my(%params) = @_;

    # FIX: allow params to be optional
    die 'missing start param in limit' if !defined($params{start});
    die 'missing start count in limit' if !defined($params{count});

    my $sql = new SQL::Interpolate::Macro(
        type => 'limit',
        start => $params{start}, count => $params{count}
    );

    # Note: we deter formatting until sql_interp since
    # this function does not know anything about which
    # SQL dialect to general SQL for.

    return $sql;
}

# Generate a sql_interp like function that is tied to a certain
# database.  This ensures that the SQL generated is compatible
# with the given database.
sub make_sql_interp
{
    my($dbh  # database handle
    ) = @_;

    my $interp = sub {
        return sql_interp($dbh, @_);
    };
    return $interp;
}

# Generate a dbi_interp like function that is tied to a certain
# database.  This ensures that the SQL generated is compatible
# with the given database.
sub make_dbi_interp
{
    my($dbh  # database handle
    ) = @_;

    my $interp = sub {
        return dbi_interp($dbh, @_);
    };
    return $interp;
}


package SQL::Interpolate::Macro;
# Macros embedded in the SQL (e.g. LIMIT macro).

sub new
{
    my($class, %params) = @_;
    my $self = {
        %params
    };
    bless $self, $class;
    return $self;
}
